@@ -3,6 +3,7 @@ import decimal
 import enum
 import inspect
 import threading
+import typing
 import weakref
 from ctypes import (
     CFUNCTYPE,
@@ -93,11 +94,11 @@ _keep_alive_objects = {}
 
 def encoding_from_annotation(f, offset=1):
     argspec = inspect.getfullargspec(inspect.unwrap(f))
-
-    encoding = [argspec.annotations.get("return", ObjCInstance), ObjCInstance, SEL]
+    hints = typing.get_type_hints(f)
+    encoding = [hints.get("return", ObjCInstance), ObjCInstance, SEL]
 
     for varname in argspec.args[offset:]:
-        encoding.append(argspec.annotations.get(varname, ObjCInstance))
+        encoding.append(hints.get(varname, ObjCInstance))
 
     return encoding
 
@@ -220,7 +221,7 @@ class ObjCMethod:
         if not convert_result:
             return result
 
-        # Convert result to python type if it is a instance or class pointer.
+        # Convert result to python type if it is an instance or class pointer.
         if self.restype is not None and issubclass(self.restype, objc_id):
             result = ObjCInstance(result)
 
@@ -260,7 +261,8 @@ class ObjCPartialMethod:
             name, order = self.methods[rest]
         except KeyError:
             raise ValueError(
-                f"No method was found starting with {self.name_start!r} and with keywords {set(kwargs)}\nKnown keywords are:\n"
+                f"No method was found starting with {self.name_start!r} and with keywords {set(kwargs)}\n"
+                f"Known keywords are:\n"
                 + "\n".join(repr(keywords) for keywords in self.methods)
             )
 
@@ -277,7 +279,7 @@ class ObjCBoundMethod:
     def __init__(self, method, receiver):
         """Initialize with a method and ObjCInstance or ObjCClass object."""
         self.method = method
-        if type(receiver) == Class:
+        if type(receiver) is Class:
             self.receiver = cast(receiver, objc_id)
         else:
             self.receiver = receiver
@@ -393,19 +395,19 @@ class objc_classmethod:
 
 
 class objc_ivar:
-    """Defines an ivar in a custom Objective-C class.
+    """Defines an ``ivar`` in a custom Objective-C class.
 
     If you want to store additional data on a custom Objective-C class, it is
-    recommended to use properties (:func:`objc_property`) instead of ivars.
+    recommended to use properties (:func:`objc_property`) instead of ``ivars``.
     Properties are a more modern and high-level Objective-C feature, which
     automatically deal with reference counting for objects, and creation of
     getters and setters.
 
-    The ivar type may be any :mod:`ctypes` type.
+    The ``ivar`` type may be any :mod:`ctypes` type.
 
-    Unlike properties, the contents of an ivar cannot be accessed or modified
-    using Python attribute syntax. Instead, the :func:`get_ivar` and
-    :func:`set_ivar` functions need to be used.
+    Unlike properties, the contents of an ``ivar`` cannot be accessed or
+    modified using Python attribute syntax. Instead, the :func:`get_ivar`
+    and :func:`set_ivar` functions need to be used.
     """
 
     def __init__(self, vartype):
@@ -437,7 +439,7 @@ class objc_property:
     the getter is called ``name`` and the setter is called ``setName:``.
 
     In a custom Objective-C class, implementations for the getter and setter are
-    also generated, which store the property's value in an ivar called
+    also generated, which store the property's value in an ``ivar`` called
     ``_name``. If the property has an object type, the generated setter keeps
     the stored object retained, and releases it when it is replaced.
 
@@ -966,9 +968,9 @@ class ObjCInstance:
 
     def __del__(self):
         """Release the corresponding objc instance if we own it, i.e., if it
-        was returned by by a method starting with 'alloc', 'new', 'copy', or
-        'mutableCopy' and it wasn't already explicitly released by calling
-        :meth:`release` or :meth:`autorelease`."""
+        was returned by a method starting with :meth:`alloc`, :meth:`new`,
+        :meth:`copy`, or :meth:`mutableCopy` and it wasn't already explicitly
+        released by calling :meth:`release` or :meth:`autorelease`."""
         if self._needs_release:
             send_message(self, "release", restype=objc_id, argtypes=[])
 
@@ -1010,7 +1012,7 @@ class ObjCInstance:
         returned using its getter method. An attribute is considered a property
         if any of the following are true:
 
-        * A property with the name is present on the class (i. e. declared using
+        * A property with the name is present on the class (i.e. declared using
           ``@property`` in the source code)
         * There is both a getter and setter method for the name
         * The name has been declared as a property using
@@ -1034,7 +1036,7 @@ class ObjCInstance:
         The "interleaved" syntax is usually preferred, since it looks more
         similar to normal Objective-C syntax. However, the "flat" syntax is also
         fully supported. Certain method names require the "flat" syntax, for
-        example if two arguments have the same label (e. g.
+        example if two arguments have the same label (e.g.
         ``performSelector:withObject:withObject:``), which is not supported by
         Python's keyword argument syntax.
 
@@ -1513,38 +1515,35 @@ class ObjCClass(ObjCInstance, type):
     def declare_property(self, name):
         """Declare the instance method ``name`` to be a property getter.
 
-        This causes the attribute named ``name`` on instances of this class to
-        be treated as a property rather than a method --- accessing it returns
-        the property's value, without requiring an explicit method call. See
-        :class:`ObjCInstance.__getattr__` for a full description of how
-        attribute access behaves for properties.
+        This causes the attribute named ``name`` on instances of this class to be
+        treated as a property rather than a method --- accessing it returns the
+        property's value, without requiring an explicit method call. See
+        :class:`ObjCInstance.__getattr__` for a full description of how attribute access
+        behaves for properties.
 
-        Most properties do not need to be declared explicitly using this method,
-        as they are detected automatically by :class:`ObjCInstance.__getattr__`.
-        This method only needs to be used for properties that are read-only and
-        don't have a ``@property`` declaration in the source code, because
-        Rubicon cannot tell such properties apart from normal zero-argument
-        methods.
+        Most properties do not need to be declared explicitly using this method, as they
+        are detected automatically by :class:`ObjCInstance.__getattr__`. This method
+        only needs to be used for properties that are read-only and don't have a
+        ``@property`` declaration in the source code, because Rubicon cannot tell such
+        properties apart from normal zero-argument methods.
 
         .. note::
 
-            In the standard Apple SDKs, some properties are introduced as
-            regular methods in one system version, and then declared as
-            properties in a later system version. For example, the
-            ``description`` method/property of :class:`NSObject` was declared as
-            a regular method `up to OS X 10.9
+            In the standard Apple SDKs, some properties are introduced as regular
+            methods in one system version, and then declared as properties in a later
+            system version. For example, the ``description`` method/property of
+            :class:`NSObject` was declared as a regular method `up to OS X 10.9
             <https://github.com/phracker/MacOSX-SDKs/blob/9fc3ed0ad0345950ac25c28695b0427846eea966/MacOSX10.9.sdk/usr/include/objc/NSObject.h#L40>`__,
             but changed to a property `as of OS X 10.10
             <https://github.com/phracker/MacOSX-SDKs/blob/9fc3ed0ad0345950ac25c28695b0427846eea966/MacOSX10.10.sdk/usr/include/objc/NSObject.h#L43>`__.
 
-            Such properties cause compatibility issues when accessed from
-            Rubicon: ``obj.description()`` works on 10.9 but is a
-            :class:`TypeError` on 10.10, whereas ``obj.description`` works on
-            10.10 but returns a method object on 10.9. To solve this issue, the
-            property can be declared explicitly using
-            ``NSObject.declare_property('description')``, so that it can always
-            be accessed using ``obj.description``.
-        """  # noqa: E501 # The links in the docstring above are too long to wrap to 120 chars.
+            Such properties cause compatibility issues when accessed from Rubicon:
+            ``obj.description()`` works on 10.9 but is a :class:`TypeError` on 10.10,
+            whereas ``obj.description`` works on 10.10 but returns a method object on
+            10.9. To solve this issue, the property can be declared explicitly using
+            ``NSObject.declare_property('description')``, so that it can always be
+            accessed using ``obj.description``.
+        """
 
         self.forced_properties.add(name)
 
@@ -2162,10 +2161,23 @@ class ObjCBlock:
             if not self.has_signature:
                 raise ValueError("Cannot use AUTO types for blocks without signatures")
             restype, *argtypes = ctypes_for_method_encoding(self.signature)
-        self.struct.contents.invoke.restype = ctype_for_type(restype)
-        self.struct.contents.invoke.argtypes = (objc_id,) + tuple(
+            # If the argtypes have been derived from the signature, they will include
+            # the block as the first argument.
+            block_arg = []
+        else:
+            # If the argtypes are explicitly provided, they *won't* include the
+            # first required argument - the block itself.
+            block_arg = [objc_id]
+
+        # If you set restype and argtypes on the invoke function that is in the
+        # ObjCBlockStruct, subsequent gets won't reflect those changes, because it's not
+        # a distinct Python object that ctypes can use to attach a type hint. Store the
+        # ctypes annotations, and apply them just before invocation. We're going to have
+        # to do some light type conversion in some cases anyway, so this works out well.
+        self.invoke_restype = ctype_for_type(restype)
+        self.invoke_argtypes = block_arg + [
             ctype_for_type(arg_type) for arg_type in argtypes
-        )
+        ]
 
     def __repr__(self):
         representation = f"<ObjCBlock@{hex(addressof(self.pointer))}"
@@ -2183,7 +2195,43 @@ class ObjCBlock:
         objects according to the default ``ctypes`` rules, based on the
         block's return and parameter types.
         """
-        return self.struct.contents.invoke(self.pointer, *args)
+        # If any of the arguments are structures, they may be anonymous - that is, we
+        # have a descriptor like "{=ii}", which tells us there are two integer fields,
+        # but doesn't provide a name for the structure. ctypes looks for an exact match
+        # of type names, so even if the field types of a structure provided as an
+        # argument match, ctypes will raise a TypeError.
+        #
+        # To avoid this, if an argument is a structure, and the argtype for that
+        # argument is a structure, look for the `__anonymous__` property on the argtype
+        # structure definition - this property is added automatically to structures when
+        # a structure type is constructed from a type descriptor that doesn't provide a
+        # name. If it exists, the structure has been anonymously declared; so we check
+        # that the provided argument matches the "shape" of the anonymous structure. If
+        # it matches, modify the invoke signature to match the type of the argument that
+        # was actually provided. The first argument to invoke is the block being
+        # invoked, so we can ignore that type hint.
+        for i, argtype in enumerate(self.invoke_argtypes[1:]):
+            if (
+                isinstance(args[i], Structure)
+                and issubclass(argtype, Structure)
+                and getattr(argtype, "__anonymous__", False)
+            ):
+                anon_fields = [f[1] for f in argtype._fields_]
+                arg_fields = [f[1] for f in args[i]._fields_]
+                if anon_fields != arg_fields:
+                    raise TypeError(
+                        f"Expected structure with field types {anon_fields} "
+                        f"for argument {i+1}; got {type(args[i]).__name__} "
+                        f"with field types {arg_fields}"
+                    )
+                self.invoke_argtypes[i + 1] = type(args[i])
+
+        # Apply the ctypes hints to the invoke function for the block.
+        invoke = self.struct.contents.invoke
+        invoke.restype = self.invoke_restype
+        invoke.argtypes = self.invoke_argtypes
+
+        return invoke(self.pointer, *args)
 
 
 class ObjCBlockInstance(ObjCInstance):
@@ -2218,7 +2266,8 @@ class Block:
 
         .. code-block:: python
 
-            @Block def the_block(arg: NSInteger) -> NSUInteger:
+            @Block
+            def the_block(arg: NSInteger) -> NSUInteger:
                 return abs(arg)
 
         For callables without type annotations, the parameter and return types
@@ -2245,6 +2294,7 @@ class Block:
             # so try to extract them from the function's type annotations.
 
             try:
+                hints = typing.get_type_hints(func)
                 signature = inspect.signature(func)
             except (TypeError, ValueError):
                 raise ValueError(
@@ -2252,28 +2302,28 @@ class Block:
                     "please pass return and argument types directly into Block"
                 )
 
-            if signature.return_annotation == inspect.Signature.empty:
+            try:
+                restype = hints["return"]
+            except KeyError:
                 raise ValueError(
                     "Function has no return type annotation - "
                     "please add one, or pass return and argument types directly into Block"
                 )
 
-            restype = signature.return_annotation
             argtypes = []
-            for name, param in signature.parameters.items():
-                if param.annotation == inspect.Parameter.empty:
+            for name in signature.parameters:
+                try:
+                    argtypes.append(hints[name])
+                except KeyError:
                     raise ValueError(
                         f"Function has no argument type annotation for parameter {name!r} - "
                         f"please add one, or pass return and argument types directly into Block"
                     )
 
-                argtypes.append(param.annotation)
-
         signature = tuple(ctype_for_type(tp) for tp in argtypes)
 
         restype = ctype_for_type(restype)
-
-        self.cfunc_type = CFUNCTYPE(restype, c_void_p, *signature)
+        cfunc_type = CFUNCTYPE(restype, c_void_p, *signature)
 
         self.literal = BlockLiteral()
         self.literal.isa = addressof(_NSConcreteStackBlock)
@@ -2283,8 +2333,9 @@ class Block:
             | BlockConsts.HAS_COPY_DISPOSE
         )
         self.literal.reserved = 0
-        self.cfunc_wrapper = self.cfunc_type(self.wrapper)
-        self.literal.invoke = cast(self.cfunc_wrapper, c_void_p)
+        cfunc_wrapper = cfunc_type(self.wrapper)
+        self.literal.invoke = cast(cfunc_wrapper, c_void_p)
+
         self.descriptor = BlockDescriptor()
         self.descriptor.reserved = 0
         self.descriptor.size = sizeof(BlockLiteral)
@@ -2303,7 +2354,10 @@ class Block:
         self.block = cast(byref(self.literal), objc_block)
         self._as_parameter_ = self.block
 
-    def wrapper(self, instance, *args):
+    def wrapper(self, block, *args):
+        # ObjC blocks take the block as the first argument when they're invoked;
+        # but since this is a wrapper around a Python object, we know the function
+        # that has to be invoked.
         return self.func(*args)
 
     def dispose_helper(self, dst):
