@@ -12,7 +12,7 @@ from asyncio import (
 )
 from ctypes import CFUNCTYPE, POINTER, Structure, c_double, c_int, c_ulong, c_void_p
 
-from .api import objc_const
+from .api import ObjCClass, objc_const
 from .runtime import load_library, objc_id
 from .types import CFIndex
 
@@ -79,6 +79,7 @@ kCFSocketWriteCallBack = 8
 kCFSocketAutomaticallyReenableReadCallBack = 1
 kCFSocketAutomaticallyReenableWriteCallBack = 8
 
+NSRunLoop = ObjCClass("NSRunLoop")
 
 ###########################################################################
 # CoreFoundation methods for async handlers
@@ -227,7 +228,7 @@ class CFSocketHandle(events.Handle):
         if self._fd not in self._loop._sockets:
             # Spurious notifications seem to be generated sometimes if you
             # CFSocketDisableCallBacks in the middle of an event.  I don't know
-            # about this FD, any more, so let's get rid of it.
+            # about this FD any more, so let's get rid of it.
             libcf.CFRunLoopRemoveSource(
                 self._loop._cfrunloop, self._src, kCFRunLoopCommonModes
             )
@@ -308,10 +309,9 @@ class CFSocketHandle(events.Handle):
     def cancel(self):
         """(Potentially) cancel the socket handle.
 
-        A socket handle can have both reader and writer components; a
-        call to cancel a socket handle will only be successfull if
-        *both* the reader and writer component have been disabled. If
-        either is still active, cancel() will be a no-op.
+        A socket handle can have both reader and writer components; a call to cancel a
+        socket handle will only be successful if *both* the reader and writer component
+        have been disabled. If either is still active, cancel() will be a no-op.
         """
         if self._reader is None and self._writer is None and self._src:
             super().cancel()
@@ -523,10 +523,10 @@ class CFEventLoop(unix_events.SelectorEventLoop):
             events._set_running_loop(self)
 
         # Start the lifecycle, but invoke it as a deferred event on the event
-        # loop. iOSLifeCycle.start() invokes libcf.CFRunLoopRun(); this ensures
-        # that a full CFRunLoop is running, not just one that responds to
+        # loop. iOSLifeCycle.start() starts the NSRunLoop; this ensures
+        # that a full NSRunLoop is running, not just one that responds to
         # iOS events. See #228 for the sort of behavior that occurs on threads
-        # if the CFRunLoop isn't started.
+        # if the NSRunLoop isn't started.
         self.call_soon(self._lifecycle.start)
 
     def call_soon(self, callback, *args, context=None):
@@ -735,7 +735,7 @@ class CFLifecycle:
 
 
 class CocoaLifecycle:
-    """A lifecycle manager for Cocoa (``NSApplication``) apps."""
+    """A life cycle manager for Cocoa (``NSApplication``) apps."""
 
     def __init__(self, application):
         self._application = application
@@ -748,10 +748,10 @@ class CocoaLifecycle:
 
 
 class iOSLifecycle:
-    """A lifecycle manager for iOS (``UIApplication``) apps."""
+    """A life cycle manager for iOS (``UIApplication``) apps."""
 
     def start(self):
-        libcf.CFRunLoopRun()
+        NSRunLoop.currentRunLoop.run()
 
     def stop(self):
         libcf.CFRunLoopStop(libcf.CFRunLoopGetMain())
