@@ -1,37 +1,26 @@
-import platform
+from __future__ import annotations
+
 import unittest
-from ctypes import Structure, c_int, c_void_p
+from ctypes import Structure, c_float, c_int, c_void_p
 
 from rubicon.objc import NSObject, ObjCBlock, ObjCClass, objc_method
 from rubicon.objc.api import Block
 from rubicon.objc.runtime import objc_block
 
 
-def m1_block_failure(test):
-    # Properties/methods returning blocks currently fail on ARM hardware.
-    # See #225 for details
-    if platform.processor().startswith("arm"):
-        return unittest.expectedFailure(test)
-    else:
-        return test
-
-
 class BlockTests(unittest.TestCase):
-    @m1_block_failure
     def test_block_property_ctypes(self):
         BlockPropertyExample = ObjCClass("BlockPropertyExample")
         instance = BlockPropertyExample.alloc().init()
         result = ObjCBlock(instance.blockProperty, c_int, c_int, c_int)(1, 2)
         self.assertEqual(result, 3)
 
-    @m1_block_failure
     def test_block_property_pytypes(self):
         BlockPropertyExample = ObjCClass("BlockPropertyExample")
         instance = BlockPropertyExample.alloc().init()
         result = ObjCBlock(instance.blockProperty, int, int, int)(1, 2)
         self.assertEqual(result, 3)
 
-    @m1_block_failure
     def test_block_delegate_method_manual_ctypes(self):
         class DelegateManualC(NSObject):
             @objc_method
@@ -44,7 +33,6 @@ class BlockTests(unittest.TestCase):
         result = instance.blockExample()
         self.assertEqual(result, 5)
 
-    @m1_block_failure
     def test_block_delegate_method_manual_pytypes(self):
         class DelegateManualPY(NSObject):
             @objc_method
@@ -57,7 +45,6 @@ class BlockTests(unittest.TestCase):
         result = instance.blockExample()
         self.assertEqual(result, 5)
 
-    @m1_block_failure
     def test_block_delegate_auto(self):
         class DelegateAuto(NSObject):
             @objc_method
@@ -70,7 +57,24 @@ class BlockTests(unittest.TestCase):
         result = instance.blockExample()
         self.assertEqual(result, 9)
 
-    @m1_block_failure
+    def test_block_delegate_manual_struct(self):
+        class BlockStruct(Structure):
+            _fields_ = [
+                ("a", c_int),
+                ("b", c_int),
+            ]
+
+        class DelegateManualStruct(NSObject):
+            @objc_method
+            def structBlockMethod_(self, block: objc_block) -> int:
+                return ObjCBlock(block, int, BlockStruct)(BlockStruct(42, 43))
+
+        BlockObjectExample = ObjCClass("BlockObjectExample")
+        delegate = DelegateManualStruct.alloc().init()
+        instance = BlockObjectExample.alloc().initWithDelegate_(delegate)
+        result = instance.structBlockExample()
+        self.assertEqual(result, 85)
+
     def test_block_delegate_auto_struct(self):
         class BlockStruct(Structure):
             _fields_ = [
@@ -89,6 +93,32 @@ class BlockTests(unittest.TestCase):
         result = instance.structBlockExample()
         self.assertEqual(result, 85)
 
+    def test_block_delegate_auto_struct_mismatch(self):
+        class BadBlockStruct(Structure):
+            _fields_ = [
+                ("a", c_float),
+                ("b", c_int),
+            ]
+
+        class BadDelegateAutoStruct(NSObject):
+            @objc_method
+            def structBlockMethod_(self, block: objc_block) -> int:
+                try:
+                    # block accepts an anonymous structure with 2 int arguments
+                    # Passing in BadBlockStruct should raise a type error because
+                    # the structure's shape doesn't match.
+                    return block(BadBlockStruct(2.71828, 43))
+                except TypeError:
+                    # Ideally, this would be raised as an ObjC error;
+                    # however, at least for now, that doesn't happen.
+                    return -99
+
+        BlockObjectExample = ObjCClass("BlockObjectExample")
+        delegate = BadDelegateAutoStruct.alloc().init()
+        instance = BlockObjectExample.alloc().initWithDelegate_(delegate)
+        result = instance.structBlockExample()
+        self.assertEqual(result, -99)
+
     def test_block_receiver(self):
         BlockReceiverExample = ObjCClass("BlockReceiverExample")
         instance = BlockReceiverExample.alloc().init()
@@ -104,11 +134,21 @@ class BlockTests(unittest.TestCase):
         self.assertEqual(values, [27])
         self.assertEqual(result, 42)
 
-    def test_block_receiver_unannotated(self):
+    def test_block_receiver_no_return_annotation(self):
         BlockReceiverExample = ObjCClass("BlockReceiverExample")
         instance = BlockReceiverExample.alloc().init()
 
-        def block(a, b):
+        def block(a: int, b: int):
+            return a + b
+
+        with self.assertRaises(ValueError):
+            instance.receiverMethod_(block)
+
+    def test_block_receiver_missing_arg_annotation(self):
+        BlockReceiverExample = ObjCClass("BlockReceiverExample")
+        instance = BlockReceiverExample.alloc().init()
+
+        def block(a: int, b) -> int:
             return a + b
 
         with self.assertRaises(ValueError):
@@ -131,7 +171,6 @@ class BlockTests(unittest.TestCase):
 
         self.assertEqual(values, [27])
 
-    @m1_block_failure
     def test_block_round_trip(self):
         BlockRoundTrip = ObjCClass("BlockRoundTrip")
         instance = BlockRoundTrip.alloc().init()
@@ -160,7 +199,6 @@ class BlockTests(unittest.TestCase):
         returned_block_2 = instance.roundTripNoArgs(block_2)
         self.assertEqual(returned_block_2(), 42)
 
-    @m1_block_failure
     def test_block_bound_method(self):
         """A bound method with type annotations can be wrapped in a block."""
 
